# -*- coding: utf-8 -*-
import base64
import hashlib
import os
import os.path
import subprocess


def subprocess_args(include_stdout=True):
    if hasattr(subprocess, 'STARTUPINFO'):
        si = subprocess.STARTUPINFO()
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        env = os.environ
    else:
        si = None
        env = None

    if include_stdout:
        ret = {'stdout': subprocess.PIPE}
    else:
        ret = {}

    ret.update({'stdin': subprocess.PIPE,
                'stderr': subprocess.PIPE,
                'startupinfo': si,
                'env': env})
    return ret


class HelperMethods:

    @staticmethod
    def get_SHA256(string):
        """
        Compute the SHA256 signature of a string.
        """
        return hashlib.sha256(string.encode("utf-8")).hexdigest()

    @staticmethod
    def I2OSP(x, xLen):
        if x > (1 << (8 * xLen)):
            return None
        Xrev = []
        for _ in range(0, xLen):
            x, m = divmod(x, 256)
            Xrev.append(m)
        return bytes(reversed(Xrev))

    @staticmethod
    def OS2IP(X):
        import binascii
        h = binascii.hexlify(X)
        return int(h, 16)

    @staticmethod
    def RSAVP1(pair, s):
        n, e = pair
        if s < 0 or n - 1 < s:
            return None
        return pow(s, e, n)

    @staticmethod
    def EMSA_PKCS1_V15_ENCODE(M, emLen):
        import hashlib
        h = hashlib.sha256()
        h.update(M)
        H = h.digest()

        T = bytes(
            [0x30, 0x31, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86, 0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x01, 0x05, 0x00, 0x04,
             0x20]) + H
        tLen = len(T)
        if emLen < tLen + 11:
            return None
        PS = bytes([0xff for _ in range(emLen - tLen - 3)])
        return b"".join([b"\x00\x01", PS, b"\x00", T])

    @staticmethod
    def RSAASSA_PKCS1_V15_VERIFY(pair, M, S):
        n, e = pair
        s = HelperMethods.OS2IP(S)
        m = HelperMethods.RSAVP1((n, e), s)
        if m is None: return False
        EM = HelperMethods.I2OSP(m, 256)
        if EM is None: return False
        EM2 = HelperMethods.EMSA_PKCS1_V15_ENCODE(M, 256)  # Can return None, but it's OK since EM is not None
        return EM == EM2

    @staticmethod
    def verify_signature(response, rsaPublicKey):
        """
        Verifies a signature from .NET RSACryptoServiceProvider.
        """

        n = HelperMethods.OS2IP(base64.b64decode(rsaPublicKey.modulus))
        e = HelperMethods.OS2IP(base64.b64decode(rsaPublicKey.exponent))

        m = base64.b64decode(response.license_key)
        r = base64.b64decode(response.signature)

        return HelperMethods.RSAASSA_PKCS1_V15_VERIFY((n, e), m, r)

    @staticmethod
    def int2base64(num):
        return base64.b64encode(int.to_bytes(num), byteorder='big')

    @staticmethod
    def base642int(string):
        return int.from_bytes(base64.b64decode((string)), byteorder='big')

    @staticmethod
    def start_process(command, v=1):

        output = subprocess.check_output(command,
                                         **subprocess_args(False))

        if v == 1:
            return output.decode('utf-8')
        elif v == 2:
            rawOutput = output.decode('utf-8')
            return rawOutput[rawOutput.index("UUID") + 4:].strip()
        else:
            raise ValueError("Version can be either 1 or 2.")

    @staticmethod
    def get_dbus_machine_id():
        try:
            with open("/etc/machine-id") as f:
                return f.read().strip()
        except:
            pass
        try:
            with open("/var/lib/dbus/machine-id") as f:
                return f.read().strip()
        except:
            pass
        return ""

    @staticmethod
    def get_inodes():
        import os
        files = ["/bin", "/etc", "/lib", "/root", "/sbin", "/usr", "/var"]
        inodes = []
        for file in files:
            try:
                inodes.append(os.stat(file).st_ino)
            except:
                pass
        return "".join([str(x) for x in inodes])

    @staticmethod
    def compute_machine_code():
        return HelperMethods.get_dbus_machine_id() + HelperMethods.get_inodes()



# -*- coding: utf-8 -*-
from pylicensemanager.internal import HelperMethods


class ActivatedMachine:
    def __init__(self, IP, Mid, Time, FriendlyName=""):
        self.IP = IP
        self.Mid = Mid

        # TODO: check if time is int, and convert to datetime in this case.
        self.Time = Time
        self.FriendlyName = FriendlyName



# -*- coding: utf-8 -*-
import platform
import random
import uuid
import sys
from pylicensemanager.internal import HelperMethods
from pylicensemanager.models import *
import json
from urllib.error import URLError, HTTPError


class Key:
    """
    generates and returns a 25 char key like: 2ILH-QWN7-2G2O-L1K2-2MNO
    courtesy of https://www.youtube.com/watch?v=IArt2Fgv644
    """

    def __init__(self, key=''):
        if key == '':
            self.key = self.generate()
        else:
            self.key = key.lower()

    def verify(self):
        score = 0
        check_digit = self.key[0]
        check_digit_count = 0
        chunks = self.key.split('-')
        for chunk in chunks:
            if len(chunk) != 4:
                return False
            for char in chunk:
                if char == check_digit:
                    check_digit_count += 1
                score += ord(char)
        if score == 1772 and check_digit_count == 5:
            return True
        return False

    def generate(self):
        key = ''
        chunk = ''
        check_digit_count = 0
        alphabet = 'abcdefghijklmnopqrstuvwxyz1234567890'
        while True:
            while len(key) < 25:
                char = random.choice(alphabet)
                key += char
                chunk += char
                if len(chunk) == 4:
                    key += '-'
                    chunk = ''
            key = key[:-1]
            if Key(key).verify():
                return key
            else:
                key = ''

    def __str__(self):
        valid = 'Invalid'
        if self.verify():
            valid = 'Valid'
        return self.key.upper()


class Helpers:

    @staticmethod
    def GetMachineCode(v=1):

        """
        Get a unique identifier for this device. If you want the machine code to be the same in .NET on Windows, you
        can set v=2. More information is available here: https://help.cryptolens.io/faq/index#machine-code-generation
        """

        if "windows" in platform.platform().lower():
            return HelperMethods.get_SHA256(
                HelperMethods.start_process(["cmd.exe", "/C", "wmic", "csproduct", "get", "uuid"], v))
        elif "mac" in platform.platform().lower() or "darwin" in platform.platform().lower():
            res = HelperMethods.start_process(["system_profiler", "SPHardwareDataType"])
            return HelperMethods.get_SHA256(res[res.index("UUID"):].strip())
        elif "linux" in platform.platform().lower():
            return HelperMethods.get_SHA256(HelperMethods.compute_machine_code())
        else:
            return HelperMethods.get_SHA256(HelperMethods.compute_machine_code())

    @staticmethod
    def IsOnRightMachine(license_key, is_floating_license=False, allow_overdraft=False, v=1):

        """
        Check if the device is registered with the license key.
        The version parameter is related to the one in GetMachineCode method.
        """

        current_mid = Helpers.GetMachineCode(v)

        if license_key.activated_machines is None:
            return False

        if is_floating_license:
            if len(license_key.activated_machines) == 1 and \
                    (license_key.activated_machines[0].Mid[9:] == current_mid or allow_overdraft and license_key.activated_machines[0].Mid[19:] == current_mid):
                return True
        else:
            for act_machine in license_key.activated_machines:
                if current_mid == act_machine.Mid:
                    return True

        return False



import json
import random
import requests
from requests.auth import HTTPBasicAuth
from pylicensemanager.methods import HelperMethods, Key, Helpers
from pylicensemanager.internal import *

host = "https://example.com"  # for example, https://example.com using your wordpress site url here is probably best
consumer_key = "ck_pRkVRxe0h73HBF9pzNf2CICiv8It8rZy"  # the consumer_key given to you when you establish your API with License Manager
consumer_secret = "cs_pRkVRxe0h73HBF9pzNf2CICiv8It8rZy"  # the consumer_secret given to you when you establish your API with License Manager
license_key = "ECHOE-SILEN-PATIE-GRACE"
# license_key = "44C9-42VW-7WDP-44ZL-VDSX"
gen_id = "1"
product_id = '14'
valid_for = '368'
times_activated_max = '5'

myjson_headers = {'Content-Type': 'application/json'}


# https://www.licensemanager.at/docs/rest-api/developer-documentation/

def list_all_licenses():
    """This route is used to retrieve all license keys from the database. Be very careful, if you do not need to use
    this API route in your productive environment then it is best to disable the route altogether via the settings
    page. The response contains an array of license key data objects.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/list-all-licenses/
    # GET Request URL /wp-json/lmfwc/v2/licenses
    url = host + "/wp-json/lmfwc/v2/licenses"

    json_response = requests.get(url, auth=HTTPBasicAuth(consumer_key, consumer_secret)).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    pass


def retrieve_license():
    """Retrieves a single license key by the license key string itself. The response contains the queried license key
    data object.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/retrieve-a-license/
    # GET Request Url /wp-json/lmfwc/v2/licenses/FOO-FIGHTERS
    url = host + "/wp-json/lmfwc/v2/licenses/" + license_key

    json_response = requests.get(url, auth=HTTPBasicAuth(consumer_key, consumer_secret)).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))

    # Parse license information from json url
    license_request_status = json_response_dict["success"]
    license_request_id = json_response_dict["data"]["id"]
    license_request_orderid = json_response_dict["data"]["orderId"]
    license_request_productId = json_response_dict["data"]["productId"]
    license_request_userId = json_response_dict["data"]["userId"]
    license_request_licenseKey = json_response_dict["data"]["licenseKey"]
    license_request_expiresAt = json_response_dict["data"]["expiresAt"]
    license_request_validFor = json_response_dict["data"]["validFor"]
    license_request_source = json_response_dict["data"]["source"]
    license_request_status = json_response_dict["data"]["status"]
    license_request_timesActivated = json_response_dict["data"]["timesActivated"]
    license_request_timesActivatedMax = json_response_dict["data"]["timesActivatedMax"]
    license_request_createdAt = json_response_dict["data"]["createdAt"]
    license_request_updatedAt = json_response_dict["data"]["updatedAt"]
    license_request_updatedBy = json_response_dict["data"]["updatedBy"]

    # Print parsed results to stdout
    print("license_request_status : %s" % license_request_status)
    print("license_request_id : %s" % license_request_id)
    print("license_request_orderid : %r" % license_request_orderid)
    print("license_request_productId : %s" % license_request_productId)
    print("license_request_userId : %s" % license_request_userId)
    print("license_request_licenseKey : %s" % license_request_licenseKey)
    print("license_request_expiresAt : %s" % license_request_expiresAt)
    print("license_request_validFor : %r" % license_request_validFor)
    print("license_request_source : %s" % license_request_source)
    print("license_request_status : %s" % license_request_status)
    print("license_request_timesActivated : %s" % license_request_timesActivated)
    print("license_request_timesActivatedMax : %s" % license_request_timesActivatedMax)
    print("license_request_createdAt : %r" % license_request_createdAt)
    print("license_request_updatedAt : %s" % license_request_updatedAt)
    print("license_request_updatedBy : %s" % license_request_updatedBy)
    return json_response_dict
    pass


def create_license():
    """Creates a new license key with the given parameters from the request body. It is possible to leave out certain
    keys, or explicitly set them to “null”. The response will contain the newly created license key data object.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/create-a-license/
    # POST Request Url /wp-json/lmfwc/v2/licenses
    url = host + "/wp-json/lmfwc/v2/licenses"

    license_generated = Key()

    # Setup fallback if no license_key provided to generate one.
    key = license_key if bool(license_key) is not False else license_generated

    payload = {
        "product_id": str(product_id),
        "license_key": str(key),
        "valid_for": str(valid_for),
        "status": "active",
        "times_activated_max": str(times_activated_max)
    }

    payld = json.dumps(payload)

    json_response = requests.post(url, data=payld, auth=HTTPBasicAuth(consumer_key, consumer_secret),
                                  headers=myjson_headers).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    # print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response
    pass


def update_license():
    """Performs an update of the license key. The request will not update key values that aren’t present in the
    request body, however if they are present their value will be updated, even if it’s a null value. The request
    will return the updated license key data object.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/update-a-license/
    # Put Request Url /wp-json/lmfwc/v2/licenses/THE-PRETENDER
    url = host + "/wp-json/lmfwc/v2/licenses/" + license_key

    payload = {
        "product_id": str(product_id),
        "license_key": str(license_key),
        "valid_for": str(valid_for),
        "status": "active",
        "times_activated_max": str(times_activated_max)
    }

    payld = json.dumps(payload)
    json_response = requests.put(url, data=payld, auth=HTTPBasicAuth(consumer_key, consumer_secret),
                                 headers=myjson_headers).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    # print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response
    pass


def activate_license():
    """Increases the “times_activated” value by one (1). The plugin will check if there is a “times_activated_max”
    value, if there is and if the “times_activated” value has not reached the limit set by “times_activated_max”,
    then the “times_activated” will be incremented by 1, and the updated license key data object will be returned.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/activate-a-license/
    # GET Request Url /wp-json/lmfwc/v2/licenses/activate/THE-PRETENDER
    url = host + "/wp-json/lmfwc/v2/licenses/activate/" + license_key

    json_response = requests.get(url, auth=HTTPBasicAuth(consumer_key, consumer_secret)).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response
    pass


def activate_license_uuid():
    """Increases the “times_activated” value by one (1). The plugin will check if there is a “times_activated_max”
    value, if there is and if the “times_activated” value has not reached the limit set by “times_activated_max”,
    then the “times_activated” will be incremented by 1, and the updated license key data object will be returned. This adds json encoded uuid payload.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/activate-a-license/
    # https://www.licensemanager.at/docs/tutorials-how-to/additional-rest-api-validation/
    # GET Request Url /wp-json/lmfwc/v2/licenses/activate/THE-PRETENDER
    url = host + "/wp-json/lmfwc/v2/licenses/activate/" + license_key

    # json_response = requests.get(url, auth=HTTPBasicAuth(consumer_key, consumer_secret)).text
    payload = {
        "uuid": str(Helpers.GetMachineCode())
    }

    payld = json.dumps(payload)

    json_response = requests.get(url, data=payld, auth=HTTPBasicAuth(consumer_key, consumer_secret),
                                 headers=myjson_headers).text

    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response
    pass


def deactivate_license():
    """Decreases the “times_activated” value by one (1). The plugin will check if the current value of
    “times_activated” is one (1) or greater. If so, the plugin will decrement the “times_activated” value by one and
    the updated license key will be returned. It is not possible to deactivate a license key whose “times_activated”
    is null or already zero (0). The response will contain the updated license key data object.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/deactivate-a-license/
    # GET Request Url /wp-json/lmfwc/v2/licenses/deactivate/THE-PRETENDER
    url = host + "/wp-json/lmfwc/v2/licenses/deactivate/" + license_key

    json_response = requests.get(url, auth=HTTPBasicAuth(consumer_key, consumer_secret)).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response
    pass


def validate_license():
    """Checks the current activation status of a license key. The response will contain the number of activations,
    the maximum activation count, and the remaining activations.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/validate-a-license/
    # GET Request Url /wp-json/lmfwc/v2/licenses/validate/THE-PRETENDER
    url = host + "/wp-json/lmfwc/v2/licenses/validate/" + license_key

    json_response = requests.get(url, auth=HTTPBasicAuth(consumer_key, consumer_secret)).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response
    pass


def list_all_generators():
    """Retrieves all currently available generators. The response will contain an array of generator data objects.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/list-all-generators/
    # GET Request Url /wp-json/lmfwc/v2/generators
    url = host + "/wp-json/lmfwc/v2/generators"

    json_response = requests.get(url, auth=HTTPBasicAuth(consumer_key, consumer_secret)).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response
    pass


def retrieve_generator():
    """Retrieves a single generator by its ID. The response contains the generator data object.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/retrieve-a-generator/
    # GET Request Url /wp-json/lmfwc/v2/generators/2
    url = host + "/wp-json/lmfwc/v2/generators/" + str(gen_id)

    json_response = requests.get(url, auth=HTTPBasicAuth(consumer_key, consumer_secret)).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response
    pass


def create_generator():
    """Creates a new generator. The response contains the newly created generator data object.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/create-a-generator/
    # POST Request Url /wp-json/lmfwc/v2/generators
    url = host + "/wp-json/lmfwc/v2/generators"

    payload = {
        "name": "Generator created by the API",
        "charset": "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789",
        "chunks": 5,
        "chunk_length": 4,
        "times_activated_max": 2,
        "separator": "-",
        "prefix": "null",
        "suffix": "null",
        "expires_in": "null"
    }

    payld = json.dumps(payload)

    json_response = requests.post(url, data=payld, auth=HTTPBasicAuth(consumer_key, consumer_secret),
                                  headers=myjson_headers).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    # print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response

    pass


def update_generator():
    """Updates a generator by its ID. The response contains the updated generator data object.
    """
    # https://www.licensemanager.at/docs/rest-api/developer-documentation/update-a-generator/
    # Put Request Url /wp-json/lmfwc/v2/generators/4
    url = host + "/wp-json/lmfwc/v2/generators/" + str(gen_id)

    payload = {
        "name": "Generator updated by the API",
        "charset": "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789",
        "chunks": 5,
        "chunk_length": 4,
        "times_activated_max": 5,
        "separator": "-",
        "prefix": "null",
        "suffix": "null",
        "expires_in": "null"
    }

    payld = json.dumps(payload)

    json_response = requests.post(url, data=payld, auth=HTTPBasicAuth(consumer_key, consumer_secret),
                                  headers=myjson_headers).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    # print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    return json_response

    pass


def verify_account_key(host, account_key_to_check):
    # original credits to: https://wordpress.org/support/users/processzip/ from
    # https://wordpress.org/support/topic/python-rest-api-example/
    url = host + "/wp-json/lmfwc/v2/licenses/validate/" + account_key_to_check

    json_response = requests.get(url, auth=HTTPBasicAuth(consumer_key, consumer_secret)).text
    # print(json_response)
    json_response_dict = json.loads(str(json_response))
    print(json_response_dict)
    print(json.dumps(json_response_dict, indent=4))
    try:
        if json_response_dict['success']:
            print('License verification Success :)')
            return True
    except:
        print('License verification failed!')
        return False


# How to call a function and test.
# list_all_licenses()
# retrieve_license()
# create_license()
# update_license()
# activate_license()
# deactivate_license()
# validate_license()
# list_all_generators()
# retrieve_generator()
# create_generator()
# update_generator()

# Try some keys
verify_account_key(host, "")
verify_account_key(host, "")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pylicensemanager']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.24.0,<3.0.0']

setup_kwargs = {
    'name': 'pylicensemanager',
    'version': '0.1.5',
    'description': 'PyLicenseManager is a Python client library for the License Manager for Woocommerce Rest API.',
    'long_description': '# Pylicensemanager\n\nPyLicenseManager is a Python client library for the License Manager for Woocommerce Rest API.\n\n## Installation\n\nUse the package manager [pip](https://pypi.org/) to install pylicensemanager.\n\n`\npip install pylicensemanager\n`\n\n## Usage\n\n```\nimport pylicensemanager\n```\n\nReview documentation for more information: https://www.licensemanager.at/docs/rest-api/\n\n## Contributing\nPull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.\n\nPlease make sure to update tests as appropriate.\n\n## License\n[MIT](https://choosealicense.com/licenses/mit/)',
    'author': 'Michael Ramsey',
    'author_email': 'mike@hackerdise.me',
    'maintainer': 'Michael Ramsey',
    'maintainer_email': 'mike@hackerdise.me',
    'url': 'https://www.licensemanager.at/docs/rest-api/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
